@@ -8,14 +8,15 @@ use std::time::SystemTime;
 
 use tracing::instrument;
 
+use crate::core::dag::Dag;
+use crate::core::effects::Effects;
 use crate::core::eventlog::{EventLogDb, EventReplayer};
 use crate::core::formatting::printable_styled_string;
-use crate::core::metadata::{
-    BranchesProvider, CommitMessageProvider, CommitOidProvider, DifferentialRevisionProvider,
-    ObsolescenceExplanationProvider, RelativeTimeProvider,
+use crate::core::node_descriptors::{
+    BranchesDescriptor, CommitMessageDescriptor, CommitOidDescriptor,
+    DifferentialRevisionDescriptor, ObsolescenceExplanationDescriptor, RelativeTimeDescriptor,
 };
-use crate::git::{Dag, Repo};
-use crate::tui::Effects;
+use crate::git::{GitRunInfo, Repo};
 
 pub use graph::{make_smartlog_graph, SmartlogGraph};
 pub use render::{render_graph, SmartlogOptions};
@@ -25,18 +26,21 @@ mod graph {
     use std::convert::TryFrom;
     use std::ops::Deref;
 
+    use eden_dag::DagAlgorithm;
     use tracing::instrument;
 
+    use crate::core::dag::{commit_set_to_vec, CommitSet, Dag};
+    use crate::core::effects::{Effects, OperationType};
     use crate::core::eventlog::{EventCursor, EventReplayer};
-    use crate::git::{Commit, CommitSet};
-    use crate::git::{Dag, NonZeroOid, Repo};
-    use crate::tui::{Effects, OperationType};
+    use crate::core::node_descriptors::NodeObject;
+    use crate::git::Commit;
+    use crate::git::{NonZeroOid, Repo};
 
     /// Node contained in the smartlog commit graph.
     #[derive(Debug)]
     pub struct Node<'repo> {
         /// The underlying commit object.
-        pub commit: Commit<'repo>,
+        pub object: NodeObject<'repo>,
 
         /// The OID of the parent node in the smartlog commit graph.
         ///
@@ -75,6 +79,24 @@ mod graph {
         nodes: HashMap<NonZeroOid, Node<'repo>>,
     }
 
+    impl<'repo> SmartlogGraph<'repo> {
+        /// Get a list of commits stored in the graph.
+        /// Returns commits in descending commit time order.
+        pub fn get_commits(&self) -> Vec<Commit<'repo>> {
+            let mut commits = self
+                .nodes
+                .values()
+                .filter_map(|node| match &node.object {
+                    NodeObject::Commit { commit } => Some(commit.clone()),
+                    NodeObject::GarbageCollected { oid: _ } => None,
+                })
+                .collect::<Vec<Commit<'repo>>>();
+            commits.sort_by_key(|commit| (commit.get_committer().get_time(), commit.get_oid()));
+            commits.reverse();
+            commits
+        }
+    }
+
     impl std::fmt::Debug for SmartlogGraph<'_> {
         fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
             write!(f, "<CommitGraph len={}>", self.nodes.len())
@@ -120,18 +142,18 @@ mod graph {
                     let vertex = vertex?;
                     let oid = NonZeroOid::try_from(vertex.clone())?;
 
-                    let commit = match repo.find_commit(oid)? {
-                        Some(commit) => commit,
+                    let object = match repo.find_commit(oid)? {
+                        Some(commit) => NodeObject::Commit { commit },
                         None => {
-                            // This commit may have been garbage collected.
-                            continue;
+                            // Assume that this commit was garbage collected.
+                            NodeObject::GarbageCollected { oid }
                         }
                     };
 
                     result.insert(
-                        commit.get_oid(),
+                        oid,
                         Node {
-                            commit,
+                            object,
                             parent: None,         // populated below
                             children: Vec::new(), // populated below
                             is_main: public_commits.contains(&vertex)?,
@@ -144,17 +166,25 @@ mod graph {
         };
 
         // Find immediate parent-child links.
-        let links: Vec<(NonZeroOid, NonZeroOid)> = graph
-            .iter()
-            .filter(|(_child_oid, node)| !node.is_main)
-            .flat_map(|(child_oid, node)| {
-                node.commit
-                    .get_parent_oids()
-                    .into_iter()
-                    .filter(|parent_oid| graph.contains_key(parent_oid))
-                    .map(move |parent_oid| (*child_oid, parent_oid))
-            })
-            .collect();
+        let links: Vec<(NonZeroOid, NonZeroOid)> = {
+            let non_main_node_oids =
+                graph.iter().filter_map(
+                    |(child_oid, node)| if !node.is_main { Some(child_oid) } else { None },
+                );
+
+            let mut links = Vec::new();
+            for child_oid in non_main_node_oids {
+                let parent_vertexes = dag.query().parents(CommitSet::from(*child_oid))?;
+                let parent_oids = commit_set_to_vec(&parent_vertexes)?;
+                for parent_oid in parent_oids {
+                    if graph.contains_key(&parent_oid) {
+                        links.push((*child_oid, parent_oid))
+                    }
+                }
+            }
+            links
+        };
+
         for (child_oid, parent_oid) in links.iter() {
             graph.get_mut(child_oid).unwrap().parent = Some(*parent_oid);
             graph.get_mut(parent_oid).unwrap().children.push(*child_oid);
@@ -166,9 +196,17 @@ mod graph {
     /// Sort children nodes of the commit graph in a standard order, for determinism
     /// in output.
     fn sort_children(graph: &mut SmartlogGraph) {
-        let commit_times: HashMap<NonZeroOid, git2::Time> = graph
+        let commit_times: HashMap<NonZeroOid, Option<git2::Time>> = graph
             .iter()
-            .map(|(oid, node)| (*oid, node.commit.get_time()))
+            .map(|(oid, node)| {
+                (
+                    *oid,
+                    match &node.object {
+                        NodeObject::Commit { commit } => Some(commit.get_time()),
+                        NodeObject::GarbageCollected { oid: _ } => None,
+                    },
+                )
+            })
             .collect();
         for node in graph.nodes.values_mut() {
             node.children
@@ -221,13 +259,15 @@ mod render {
 
     use cursive::theme::Effect;
     use cursive::utils::markup::StyledString;
+    use eden_dag::DagAlgorithm;
     use tracing::instrument;
 
+    use crate::core::dag::{CommitSet, CommitVertex, Dag};
+    use crate::core::effects::Effects;
     use crate::core::formatting::set_effect;
     use crate::core::formatting::{Glyphs, StyledStringBuilder};
-    use crate::core::metadata::{render_commit_metadata, CommitMetadataProvider};
-    use crate::git::{Dag, NonZeroOid, Repo};
-    use crate::tui::Effects;
+    use crate::core::node_descriptors::{render_node_descriptors, NodeDescriptor};
+    use crate::git::{NonZeroOid, Repo};
 
     use super::graph::SmartlogGraph;
 
@@ -285,20 +325,20 @@ mod render {
         root_commit_oids
     }
 
-    #[instrument(skip(commit_metadata_providers, graph))]
+    #[instrument(skip(commit_descriptors, graph))]
     fn get_child_output(
         glyphs: &Glyphs,
         graph: &SmartlogGraph,
         root_oids: &[NonZeroOid],
-        commit_metadata_providers: &mut [&mut dyn CommitMetadataProvider],
+        commit_descriptors: &mut [&mut dyn NodeDescriptor],
         head_oid: Option<NonZeroOid>,
         current_oid: NonZeroOid,
         last_child_line_char: Option<&str>,
     ) -> eyre::Result<Vec<StyledString>> {
         let current_node = &graph[&current_oid];
-        let is_head = Some(current_node.commit.get_oid()) == head_oid;
+        let is_head = Some(current_oid) == head_oid;
 
-        let text = render_commit_metadata(&current_node.commit, commit_metadata_providers)?;
+        let text = render_node_descriptors(&current_node.object, commit_descriptors)?;
         let cursor = match (current_node.is_main, current_node.is_obsolete, is_head) {
             (false, false, false) => glyphs.commit_visible,
             (false, false, true) => glyphs.commit_visible_head,
@@ -356,7 +396,7 @@ mod render {
                 glyphs,
                 graph,
                 root_oids,
-                commit_metadata_providers,
+                commit_descriptors,
                 head_oid,
                 *child_oid,
                 None,
@@ -383,11 +423,12 @@ mod render {
     }
 
     /// Render a pretty graph starting from the given root OIDs in the given graph.
-    #[instrument(skip(commit_metadata_providers, graph))]
+    #[instrument(skip(commit_descriptors, graph))]
     fn get_output(
         glyphs: &Glyphs,
+        dag: &Dag,
         graph: &SmartlogGraph,
-        commit_metadata_providers: &mut [&mut dyn CommitMetadataProvider],
+        commit_descriptors: &mut [&mut dyn NodeDescriptor],
         head_oid: Option<NonZeroOid>,
         root_oids: &[NonZeroOid],
     ) -> eyre::Result<Vec<StyledString>> {
@@ -398,18 +439,19 @@ mod render {
         // This returns `true` in strictly more cases than checking `graph`,
         // since there may be links between adjacent main branch commits which
         // are not reflected in `graph`.
-        let has_real_parent = |oid: NonZeroOid, parent_oid: NonZeroOid| -> bool {
-            graph[&oid]
-                .commit
-                .get_parent_oids()
-                .into_iter()
-                .any(|parent_oid2| parent_oid2 == parent_oid)
+        let has_real_parent = |oid: NonZeroOid, parent_oid: NonZeroOid| -> eyre::Result<bool> {
+            let parents = dag.query().parents(CommitSet::from(oid))?;
+            let result = parents.contains(&CommitVertex::from(parent_oid))?;
+            Ok(result)
         };
 
         for (root_idx, root_oid) in root_oids.iter().enumerate() {
-            let root_node = &graph[root_oid];
-            if root_node.commit.get_parent_count() > 0 {
-                let line = if root_idx > 0 && has_real_parent(*root_oid, root_oids[root_idx - 1]) {
+            if !dag
+                .query()
+                .parents(CommitSet::from(*root_oid))?
+                .is_empty()?
+            {
+                let line = if root_idx > 0 && has_real_parent(*root_oid, root_oids[root_idx - 1])? {
                     StyledString::plain(glyphs.line.to_owned())
                 } else {
                     StyledString::plain(glyphs.vertical_ellipsis.to_owned())
@@ -426,7 +468,7 @@ mod render {
                     None
                 } else {
                     let next_root_oid = root_oids[root_idx + 1];
-                    if has_real_parent(next_root_oid, *root_oid) {
+                    if has_real_parent(next_root_oid, *root_oid)? {
                         Some(glyphs.line)
                     } else {
                         Some(glyphs.vertical_ellipsis)
@@ -438,7 +480,7 @@ mod render {
                 glyphs,
                 graph,
                 root_oids,
-                commit_metadata_providers,
+                commit_descriptors,
                 head_oid,
                 *root_oid,
                 last_child_line_char,
@@ -450,20 +492,21 @@ mod render {
     }
 
     /// Render the smartlog graph and write it to the provided stream.
-    #[instrument(skip(commit_metadata_providers, graph))]
+    #[instrument(skip(commit_descriptors, graph))]
     pub fn render_graph(
         effects: &Effects,
         repo: &Repo,
         dag: &Dag,
         graph: &SmartlogGraph,
         head_oid: Option<NonZeroOid>,
-        commit_metadata_providers: &mut [&mut dyn CommitMetadataProvider],
+        commit_descriptors: &mut [&mut dyn NodeDescriptor],
     ) -> eyre::Result<Vec<StyledString>> {
         let root_oids = split_commit_graph_by_roots(effects, repo, dag, graph);
         let lines = get_output(
             effects.get_glyphs(),
+            dag,
             graph,
-            commit_metadata_providers,
+            commit_descriptors,
             head_oid,
             &root_oids,
         )?;
@@ -481,12 +524,16 @@ mod render {
 
 /// Display a nice graph of commits you've recently worked on.
 #[instrument]
-pub fn smartlog(effects: &Effects, options: &SmartlogOptions) -> eyre::Result<()> {
+pub fn smartlog(
+    effects: &Effects,
+    git_run_info: &GitRunInfo,
+    options: &SmartlogOptions,
+) -> eyre::Result<()> {
     let SmartlogOptions {
         show_hidden_commits,
     } = options;
 
-    let repo = Repo::from_current_dir()?;
+    let repo = Repo::from_dir(&git_run_info.working_directory)?;
     let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
     let event_log_db = EventLogDb::new(&conn)?;
@@ -516,15 +563,15 @@ pub fn smartlog(effects: &Effects, options: &SmartlogOptions) -> eyre::Result<()
         &graph,
         references_snapshot.head_oid,
         &mut [
-            &mut CommitOidProvider::new(true)?,
-            &mut RelativeTimeProvider::new(&repo, SystemTime::now())?,
-            &mut ObsolescenceExplanationProvider::new(
+            &mut CommitOidDescriptor::new(true)?,
+            &mut RelativeTimeDescriptor::new(&repo, SystemTime::now())?,
+            &mut ObsolescenceExplanationDescriptor::new(
                 &event_replayer,
                 event_replayer.make_default_cursor(),
             )?,
-            &mut BranchesProvider::new(&repo, &references_snapshot)?,
-            &mut DifferentialRevisionProvider::new(&repo)?,
-            &mut CommitMessageProvider::new()?,
+            &mut BranchesDescriptor::new(&repo, &references_snapshot)?,
+            &mut DifferentialRevisionDescriptor::new(&repo)?,
+            &mut CommitMessageDescriptor::new()?,
         ],
     )?;
     for line in lines {
