@@ -10,10 +10,12 @@ use eyre::Context;
 use os_str_bytes::OsStrBytes;
 use tracing::warn;
 
+use crate::core::effects::Effects;
 use crate::core::eventlog::EventTransactionId;
 use crate::core::formatting::{printable_styled_string, Pluralize};
-use crate::git::{GitRunInfo, MaybeZeroOid, NonZeroOid, Repo, ResolvedReferenceInfo};
-use crate::tui::Effects;
+use crate::git::{
+    check_out_commit, GitRunInfo, MaybeZeroOid, NonZeroOid, Repo, ResolvedReferenceInfo,
+};
 
 use super::plan::RebasePlan;
 
@@ -258,10 +260,12 @@ pub fn check_out_updated_head(
         }
     };
 
-    let result = git_run_info.run(
+    let result = check_out_commit(
         effects,
+        git_run_info,
         Some(event_tx_id),
-        &[OsStr::new("checkout"), &checkout_target],
+        Some(&checkout_target),
+        &[] as &[&OsStr],
     )?;
     Ok(result)
 }
@@ -316,6 +320,7 @@ mod in_memory {
     use tracing::{instrument, warn};
 
     use crate::commands::gc::mark_commit_reachable;
+    use crate::core::effects::Effects;
     use crate::core::formatting::printable_styled_string;
     use crate::core::rewrite::execute::check_out_updated_head;
     use crate::core::rewrite::move_branches;
@@ -323,7 +328,6 @@ mod in_memory {
     use crate::git::{
         CherryPickFastError, CherryPickFastOptions, GitRunInfo, MaybeZeroOid, NonZeroOid, Repo,
     };
-    use crate::tui::Effects;
 
     use super::{ExecuteRebasePlanOptions, MergeConflictInfo};
 
@@ -712,7 +716,6 @@ mod in_memory {
 }
 
 mod on_disk {
-    use std::borrow::Cow;
     use std::ffi::OsStr;
     use std::fmt::Write;
 
@@ -720,9 +723,9 @@ mod on_disk {
     use os_str_bytes::OsStrBytes;
     use tracing::instrument;
 
+    use crate::core::effects::{Effects, OperationType};
     use crate::core::rewrite::plan::RebasePlan;
-    use crate::git::{GitRunInfo, MaybeZeroOid, Repo};
-    use crate::tui::{Effects, OperationType};
+    use crate::git::{GitRunInfo, Repo};
 
     use super::ExecuteRebasePlanOptions;
 
@@ -781,36 +784,29 @@ mod on_disk {
         std::fs::write(&interactive_file_path, "")
             .wrap_err_with(|| format!("Writing interactive to: {:?}", &interactive_file_path))?;
 
-        if head_info.oid.is_some() {
-            let repo_head_file_path = repo.get_path().join("HEAD");
+        if let Some(head_oid) = head_info.oid {
             let orig_head_file_path = repo.get_path().join("ORIG_HEAD");
-            std::fs::copy(&repo_head_file_path, &orig_head_file_path)
-                .wrap_err_with(|| format!("Copying `HEAD` to: {:?}", &orig_head_file_path))?;
+            std::fs::write(&orig_head_file_path, head_oid.to_string())
+                .wrap_err_with(|| format!("Writing `ORIG_HEAD` to: {:?}", &orig_head_file_path))?;
 
             // Confusingly, there is also a file at
             // `.git/rebase-merge/orig-head` (different from `.git/ORIG_HEAD`),
-            // which stores only the OID of the original `HEAD` commit.  It's
-            // used by Git to rebase the originally-checked out branch, or to
-            // check out the old branch if the rebase is aborted.
-            let rebase_orig_head_oid: MaybeZeroOid = head_info.oid.into();
+            // which seems to store the same thing.
             let rebase_orig_head_file_path = rebase_state_dir.join("orig-head");
-            std::fs::write(
-                &rebase_orig_head_file_path,
-                rebase_orig_head_oid.to_string(),
-            )
-            .wrap_err_with(|| {
-                format!("Writing `orig-head` to: {:?}", &rebase_orig_head_file_path)
-            })?;
+            std::fs::write(&rebase_orig_head_file_path, head_oid.to_string()).wrap_err_with(
+                || format!("Writing `orig-head` to: {:?}", &rebase_orig_head_file_path),
+            )?;
 
-            // `head-name` appears to be purely for UX concerns. Git will warn if the
-            // file isn't found.
+            // `head-name` contains the name of the branch which will be reset
+            // to point to the OID contained in `orig-head` when the rebase is
+            // aborted.
             let head_name_file_path = rebase_state_dir.join("head-name");
             std::fs::write(
                 &head_name_file_path,
                 head_info
-                    .get_branch_name()?
-                    .map(Cow::Owned)
-                    .unwrap_or_else(|| Cow::Borrowed(OsStr::new("detached HEAD")))
+                    .reference_name
+                    .as_deref()
+                    .unwrap_or_else(|| OsStr::new("detached HEAD"))
                     .to_raw_bytes(),
             )
             .wrap_err_with(|| format!("Writing head-name to: {:?}", &head_name_file_path))?;
