@@ -8,17 +8,23 @@ use std::process::{Command, ExitStatus, Stdio};
 use std::sync::Arc;
 use std::thread::{self, JoinHandle};
 
+use cursive::theme::BaseColor;
+use cursive::utils::markup::StyledString;
 use eyre::{eyre, Context};
 use itertools::Itertools;
 use os_str_bytes::OsStrBytes;
 use tracing::instrument;
 
+use crate::commands::smartlog::smartlog;
 use crate::core::config::get_core_hooks_path;
+use crate::core::effects::{Effects, OperationType};
 use crate::core::eventlog::{EventTransactionId, BRANCHLESS_TRANSACTION_ID_ENV_VAR};
+use crate::core::formatting::printable_styled_string;
 use crate::git::repo::Repo;
-use crate::tui::{Effects, OperationType};
 use crate::util::get_sh;
 
+use super::CategorizedReferenceName;
+
 /// Path to the `git` executable on disk to be executed.
 #[derive(Clone)]
 pub struct GitRunInfo {
@@ -42,6 +48,37 @@ impl std::fmt::Debug for GitRunInfo {
     }
 }
 
+pub struct GitRunOpts {
+    /// If set, a non-zero exit code will be treated as an error.
+    treat_git_failure_as_error: bool,
+}
+
+impl Default for GitRunOpts {
+    fn default() -> Self {
+        Self {
+            treat_git_failure_as_error: true,
+        }
+    }
+}
+
+pub struct GitRunResult {
+    pub exit_code: i32,
+    pub stdout: Vec<u8>,
+    pub stderr: Vec<u8>,
+}
+
+impl std::fmt::Debug for GitRunResult {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(
+            f,
+            "<GitRunResult exit_code={:?} stdout={:?} stderr={:?}>",
+            self.exit_code,
+            String::from_utf8_lossy(&self.stdout),
+            String::from_utf8_lossy(&self.stderr),
+        )
+    }
+}
+
 impl GitRunInfo {
     fn spawn_writer_thread<
         InputStream: Read + Send + 'static,
@@ -154,20 +191,29 @@ impl GitRunInfo {
         repo: &Repo,
         event_tx_id: Option<EventTransactionId>,
         args: &[&str],
-    ) -> eyre::Result<String> {
+        opts: GitRunOpts,
+    ) -> eyre::Result<GitRunResult> {
         let GitRunInfo {
             path_to_git,
             working_directory,
             env,
         } = self;
+        let GitRunOpts {
+            treat_git_failure_as_error,
+        } = opts;
 
+        // Prefer running in the working copy path to the repo path, because
+        // some commands (notably `git status`) to not function correctly
+        // when run from the git repo (i.e. `.git`) path.
+        let repo_path = repo
+            .get_working_copy_path()
+            .unwrap_or_else(|| repo.get_path());
         // Technically speaking, we should be able to work with non-UTF-8 repository
         // paths. Need to make the typechecker accept it.
-        let repo_path = repo.get_path();
         let repo_path = repo_path.to_str().ok_or_else(|| {
             eyre::eyre!(
                 "Path to Git repo could not be converted to UTF-8 string: {:?}",
-                repo_path
+                repo.get_path()
             )
         })?;
 
@@ -184,9 +230,18 @@ impl GitRunInfo {
         if let Some(event_tx_id) = event_tx_id {
             command.env(BRANCHLESS_TRANSACTION_ID_ENV_VAR, event_tx_id.to_string());
         }
-        let result = command.output().wrap_err("Spawning Git subprocess")?;
-        let result =
-            String::from_utf8(result.stdout).wrap_err("Decoding stdout from Git subprocess")?;
+        let output = command.output().wrap_err("Spawning Git subprocess")?;
+        let result = GitRunResult {
+            // On Unix, if the child process was terminated by a signal, we need to call
+            // some Unix-specific functions to access the signal that terminated it. For
+            // simplicity, just return `1` in those cases.
+            exit_code: output.status.code().unwrap_or(1),
+            stdout: output.stdout,
+            stderr: output.stderr,
+        };
+        if treat_git_failure_as_error && !output.status.success() {
+            eyre::bail!("Git subprocess failed: {:?}", result);
+        }
         Ok(result)
     }
 
@@ -201,11 +256,13 @@ impl GitRunInfo {
         repo: &Repo,
         event_tx_id: Option<EventTransactionId>,
         args: &[S],
-    ) -> eyre::Result<String> {
+        opts: GitRunOpts,
+    ) -> eyre::Result<GitRunResult> {
         self.run_silent_inner(
             repo,
             event_tx_id,
             args.iter().map(AsRef::as_ref).collect_vec().as_slice(),
+            opts,
         )
     }
 
@@ -219,6 +276,9 @@ impl GitRunInfo {
         stdin: Option<OsString>,
     ) -> eyre::Result<()> {
         let hook_dir = get_core_hooks_path(repo)?;
+        if !hook_dir.exists() {
+            return Ok(());
+        }
 
         let GitRunInfo {
             // We're calling a Git hook, but not Git itself.
@@ -229,11 +289,12 @@ impl GitRunInfo {
             env,
         } = self;
         let path = {
-            let mut path_components: Vec<PathBuf> = vec![std::fs::canonicalize(&hook_dir)?];
+            let mut path_components: Vec<PathBuf> =
+                vec![std::fs::canonicalize(&hook_dir).wrap_err("Canonicalizing hook dir")?];
             if let Some(path) = env.get(OsStr::new("PATH")) {
                 path_components.extend(std::env::split_paths(path));
             }
-            std::env::join_paths(path_components)?
+            std::env::join_paths(path_components).wrap_err("Joining path components")?
         };
 
         if hook_dir.join(hook_name).exists() {
@@ -273,7 +334,8 @@ impl GitRunInfo {
             let stderr = child.stderr.take();
             let stderr_thread = self.spawn_writer_thread(stderr, effects.get_error_stream());
 
-            let _ignored: ExitStatus = child.wait()?;
+            let _ignored: ExitStatus =
+                child.wait().wrap_err("Waiting for child process to exit")?;
             stdout_thread.join().unwrap();
             stderr_thread.join().unwrap();
         }
@@ -304,8 +366,61 @@ impl GitRunInfo {
     }
 }
 
+/// Checks out the requested commit. If the operation succeeds, then displays
+/// the new smartlog. Otherwise displays a warning message.
+pub fn check_out_commit(
+    effects: &Effects,
+    git_run_info: &GitRunInfo,
+    event_tx_id: Option<EventTransactionId>,
+    target: Option<impl AsRef<OsStr>>,
+    additional_args: impl IntoIterator<Item = impl AsRef<OsStr>>,
+) -> eyre::Result<isize> {
+    let target = match target {
+        None => None,
+        Some(target) => {
+            let categorized_target = CategorizedReferenceName::new(target.as_ref());
+            Some(categorized_target.remove_prefix()?)
+        }
+    };
+
+    let additional_args: Vec<_> = additional_args.into_iter().collect();
+    let args = {
+        let mut args = vec![OsStr::new("checkout")];
+        if let Some(target) = &target {
+            args.push(target);
+        }
+        args.extend(additional_args.iter().map(|arg| arg.as_ref()));
+        args
+    };
+    let result = git_run_info.run(effects, event_tx_id, args.as_slice())?;
+
+    if result == 0 {
+        smartlog(effects, git_run_info, &Default::default())?;
+    } else {
+        writeln!(
+            effects.get_output_stream(),
+            "{}",
+            printable_styled_string(
+                effects.get_glyphs(),
+                StyledString::styled(
+                    match target {
+                        Some(target) =>
+                            format!("Failed to check out commit: {}", target.to_string_lossy()),
+                        None => "Failed to check out commit".to_string(),
+                    },
+                    BaseColor::Red.light()
+                )
+            )?
+        )?;
+    }
+    Ok(result)
+}
+
 #[cfg(test)]
 mod tests {
+    use insta::assert_debug_snapshot;
+
+    use super::{GitRunInfo, GitRunOpts};
     use crate::testing::make_git;
 
     #[test]
@@ -350,4 +465,46 @@ mod tests {
 
         Ok(())
     }
+
+    #[test]
+    fn test_run_silent_failures() -> eyre::Result<()> {
+        let git = make_git()?;
+        git.init_repo()?;
+
+        let git_run_info = GitRunInfo {
+            path_to_git: git.path_to_git.clone(),
+            working_directory: git.repo_path.clone(),
+            env: std::env::vars_os().collect(),
+        };
+
+        let result = git_run_info.run_silent(
+            &git.get_repo()?,
+            None,
+            &["some-nonexistent-command"],
+            GitRunOpts {
+                treat_git_failure_as_error: true,
+            },
+        );
+        assert_debug_snapshot!(result, @r###"
+        Err(
+            "Git subprocess failed: <GitRunResult exit_code=1 stdout=\"\" stderr=\"git: 'some-nonexistent-command' is not a git command. See 'git --help'.\\n\">",
+        )
+        "###);
+
+        let result = git_run_info.run_silent(
+            &git.get_repo()?,
+            None,
+            &["some-nonexistent-command"],
+            GitRunOpts {
+                treat_git_failure_as_error: false,
+            },
+        );
+        assert_debug_snapshot!(result, @r###"
+        Ok(
+            <GitRunResult exit_code=1 stdout="" stderr="git: 'some-nonexistent-command' is not a git command. See 'git --help'.\n">,
+        )
+        "###);
+
+        Ok(())
+    }
 }
