@@ -12,7 +12,6 @@ fn test_move_stick() -> eyre::Result<()> {
         return Ok(());
     }
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     let test1_oid = git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
@@ -88,6 +87,14 @@ fn test_move_stick() -> eyre::Result<()> {
         [2/2] Committed as: a2482074 create test4.txt
         branchless: processing 2 rewritten commits
         branchless: running command: <git-executable> checkout a248207402822b7396cabe0f1011d8a7ce7daf1b
+        :
+        O 62fc20d2 create test1.txt
+        |\
+        | o 4838e49b create test3.txt
+        | |
+        | @ a2482074 create test4.txt
+        |
+        O 96d1c37a (master) create test2.txt
         In-memory rebase succeeded.
         "###);
 
@@ -115,7 +122,6 @@ fn test_move_tree() -> eyre::Result<()> {
         return Ok(());
     }
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     let test1_oid = git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
@@ -191,7 +197,6 @@ fn test_move_with_source_not_in_smartlog() -> eyre::Result<()> {
         return Ok(());
     }
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     let test1_oid = git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
@@ -238,39 +243,45 @@ fn test_move_with_source_not_in_smartlog() -> eyre::Result<()> {
                 &test1_oid.to_string(),
             ])?;
             insta::assert_snapshot!(stdout, @r###"
-        Rebase plan: Some(
-            RebasePlan {
-                first_dest_oid: NonZeroOid(62fc20d2a290daea0d52bdc2ed2ad4be6491010e),
-                commands: [
-                    RegisterExtraPostRewriteHook,
-                    Reset {
-                        target: Oid(
-                            NonZeroOid(62fc20d2a290daea0d52bdc2ed2ad4be6491010e),
-                        ),
-                    },
-                    Pick {
-                        commit_oid: NonZeroOid(70deb1e28791d8e7dd5a1f0c871a51b91282562f),
-                    },
-                    DetectEmptyCommit {
-                        commit_oid: NonZeroOid(70deb1e28791d8e7dd5a1f0c871a51b91282562f),
-                    },
-                    Pick {
-                        commit_oid: NonZeroOid(355e173bf9c5d2efac2e451da0cdad3fb82b869a),
-                    },
-                    DetectEmptyCommit {
-                        commit_oid: NonZeroOid(355e173bf9c5d2efac2e451da0cdad3fb82b869a),
-                    },
-                ],
-            },
-        )
-        Attempting rebase in-memory...
-        [1/2] Committed as: 4838e49b create test3.txt
-        [2/2] Committed as: a2482074 create test4.txt
-        branchless: processing 1 update: branch master
-        branchless: processing 2 rewritten commits
-        branchless: running command: <git-executable> checkout master
-        In-memory rebase succeeded.
-        "###);
+            Rebase plan: Some(
+                RebasePlan {
+                    first_dest_oid: NonZeroOid(62fc20d2a290daea0d52bdc2ed2ad4be6491010e),
+                    commands: [
+                        RegisterExtraPostRewriteHook,
+                        Reset {
+                            target: Oid(
+                                NonZeroOid(62fc20d2a290daea0d52bdc2ed2ad4be6491010e),
+                            ),
+                        },
+                        Pick {
+                            commit_oid: NonZeroOid(70deb1e28791d8e7dd5a1f0c871a51b91282562f),
+                        },
+                        DetectEmptyCommit {
+                            commit_oid: NonZeroOid(70deb1e28791d8e7dd5a1f0c871a51b91282562f),
+                        },
+                        Pick {
+                            commit_oid: NonZeroOid(355e173bf9c5d2efac2e451da0cdad3fb82b869a),
+                        },
+                        DetectEmptyCommit {
+                            commit_oid: NonZeroOid(355e173bf9c5d2efac2e451da0cdad3fb82b869a),
+                        },
+                    ],
+                },
+            )
+            Attempting rebase in-memory...
+            [1/2] Committed as: 4838e49b create test3.txt
+            [2/2] Committed as: a2482074 create test4.txt
+            branchless: processing 1 update: branch master
+            branchless: processing 2 rewritten commits
+            branchless: running command: <git-executable> checkout master
+            :
+            O 62fc20d2 create test1.txt
+            |\
+            : o 96d1c37a create test2.txt
+            :
+            @ a2482074 (master) create test4.txt
+            In-memory rebase succeeded.
+            "###);
         }
 
         {
@@ -297,7 +308,6 @@ fn test_move_merge_conflict() -> eyre::Result<()> {
         return Ok(());
     }
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     let base_oid = git.commit_file("test1", 1)?;
     git.detach_head()?;
@@ -397,7 +407,6 @@ fn test_move_base() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.commit_file("test1", 1)?;
     git.detach_head()?;
@@ -444,6 +453,12 @@ fn test_move_base() -> eyre::Result<()> {
         [2/2] Committed as: cf5eb244 create test3.txt
         branchless: processing 2 rewritten commits
         branchless: running command: <git-executable> checkout master
+        :
+        @ bf0d52a6 (master) create test4.txt
+        |
+        o 44352d00 create test2.txt
+        |
+        o cf5eb244 create test3.txt
         In-memory rebase succeeded.
         "###);
     }
@@ -495,6 +510,15 @@ fn test_move_base_shared() -> eyre::Result<()> {
         [2/2] Committed as: 355e173b create test4.txt
         branchless: processing 2 rewritten commits
         branchless: running command: <git-executable> checkout 355e173bf9c5d2efac2e451da0cdad3fb82b869a
+        O f777ecc9 (master) create initial.txt
+        |
+        o 62fc20d2 create test1.txt
+        |
+        o 96d1c37a create test2.txt
+        |
+        o 70deb1e2 create test3.txt
+        |
+        @ 355e173b create test4.txt
         In-memory rebase succeeded.
         "###);
     }
@@ -522,7 +546,6 @@ fn test_move_checkout_new_head() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.commit_file("test1", 1)?;
     git.run(&["prev"])?;
@@ -554,6 +577,10 @@ fn test_move_checkout_new_head() -> eyre::Result<()> {
         [1/1] Committed as: 96d1c37a create test2.txt
         branchless: processing 1 rewritten commit
         branchless: running command: <git-executable> checkout 96d1c37a3d4363611c49f7e52186e189a04c531f
+        :
+        O 62fc20d2 (master) create test1.txt
+        |
+        @ 96d1c37a create test2.txt
         In-memory rebase succeeded.
         "###);
     }
@@ -576,7 +603,6 @@ fn test_move_branch() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.detach_head()?;
     git.commit_file("test1", 1)?;
@@ -617,6 +643,8 @@ fn test_move_branch() -> eyre::Result<()> {
         branchless: processing 1 update: branch master
         branchless: processing 1 rewritten commit
         branchless: running command: <git-executable> checkout master
+        :
+        @ 70deb1e2 (master) create test3.txt
         In-memory rebase succeeded.
         "###);
     }
@@ -690,7 +718,6 @@ fn test_move_force_in_memory() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
@@ -725,6 +752,8 @@ fn test_rebase_in_memory_updates_committer_timestamp() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
+    git.run(&["config", "branchless.restack.preserveTimestamps", "false"])?;
+
     let repo = git.get_repo()?;
 
     git.commit_file("test1", 1)?;
@@ -757,7 +786,6 @@ fn test_move_in_memory_gc() -> eyre::Result<()> {
     }
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.detach_head()?;
     let test1_oid = git.commit_file("test1", 1)?;
@@ -803,6 +831,11 @@ fn test_move_in_memory_gc() -> eyre::Result<()> {
         [1/1] Committed as: fe65c1fe create test2.txt
         branchless: processing 1 rewritten commit
         branchless: running command: <git-executable> checkout fe65c1fe15584744e649b2c79d4cf9b0d878f92e
+        O f777ecc9 (master) create initial.txt
+        |\
+        | o 62fc20d2 create test1.txt
+        |
+        @ fe65c1fe create test2.txt
         In-memory rebase succeeded.
         "###);
     }
@@ -847,7 +880,6 @@ fn test_move_main_branch_commits() -> eyre::Result<()> {
     }
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     let test1_oid = git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
@@ -871,6 +903,12 @@ fn test_move_main_branch_commits() -> eyre::Result<()> {
         branchless: processing 1 update: branch master
         branchless: processing 3 rewritten commits
         branchless: running command: <git-executable> checkout master
+        :
+        O 62fc20d2 create test1.txt
+        |\
+        : o 96d1c37a create test2.txt
+        :
+        @ 566e4341 (master) create test5.txt
         In-memory rebase succeeded.
         "###);
     }
@@ -1001,20 +1039,19 @@ fn test_move_branches_after_move() -> eyre::Result<()> {
             // There should be no branches left to restack.
             let (stdout, _stderr) = git.run(&["restack"])?;
             insta::assert_snapshot!(stdout, @r###"
-        No abandoned commits to restack.
-        No abandoned branches to restack.
-        branchless: running command: <git-executable> checkout 566e4341a4a9a930fc2bf7ccdfa168e9f266c34a
-        :
-        O 62fc20d2 create test1.txt
-        |\
-        | o 4838e49b (foo) create test3.txt
-        | |
-        | o a2482074 create test4.txt
-        | |
-        | @ 566e4341 (bar) create test5.txt
-        |
-        O 96d1c37a (master) create test2.txt
-        "###);
+            No abandoned commits to restack.
+            No abandoned branches to restack.
+            :
+            O 62fc20d2 create test1.txt
+            |\
+            | o 4838e49b (foo) create test3.txt
+            | |
+            | o a2482074 create test4.txt
+            | |
+            | @ 566e4341 (bar) create test5.txt
+            |
+            O 96d1c37a (master) create test2.txt
+            "###);
         }
     }
 
@@ -1042,6 +1079,16 @@ fn test_move_branches_after_move() -> eyre::Result<()> {
             branchless: processing 2 updates: branch bar, branch foo
             branchless: processing 3 rewritten commits
             branchless: running command: <git-executable> checkout 566e4341a4a9a930fc2bf7ccdfa168e9f266c34a
+            :
+            O 62fc20d2 create test1.txt
+            |\
+            | o 4838e49b (foo) create test3.txt
+            | |
+            | o a2482074 create test4.txt
+            | |
+            | @ 566e4341 (bar) create test5.txt
+            |
+            O 96d1c37a (master) create test2.txt
             In-memory rebase succeeded.
             "###);
         }
@@ -1066,20 +1113,19 @@ fn test_move_branches_after_move() -> eyre::Result<()> {
             // There should be no branches left to restack.
             let (stdout, _stderr) = git.run(&["restack"])?;
             insta::assert_snapshot!(stdout, @r###"
-        No abandoned commits to restack.
-        No abandoned branches to restack.
-        branchless: running command: <git-executable> checkout 566e4341a4a9a930fc2bf7ccdfa168e9f266c34a
-        :
-        O 62fc20d2 create test1.txt
-        |\
-        | o 4838e49b (foo) create test3.txt
-        | |
-        | o a2482074 create test4.txt
-        | |
-        | @ 566e4341 (bar) create test5.txt
-        |
-        O 96d1c37a (master) create test2.txt
-        "###);
+            No abandoned commits to restack.
+            No abandoned branches to restack.
+            :
+            O 62fc20d2 create test1.txt
+            |\
+            | o 4838e49b (foo) create test3.txt
+            | |
+            | o a2482074 create test4.txt
+            | |
+            | @ 566e4341 (bar) create test5.txt
+            |
+            O 96d1c37a (master) create test2.txt
+            "###);
         }
     }
 
@@ -1095,7 +1141,6 @@ fn test_move_no_reapply_upstream_commits() -> eyre::Result<()> {
     }
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.detach_head()?;
     let test1_oid = git.commit_file("test1", 1)?;
@@ -1153,14 +1198,18 @@ fn test_move_no_reapply_upstream_commits() -> eyre::Result<()> {
         branchless: processing checkout
         "###);
             insta::assert_snapshot!(stdout, @r###"
-        Attempting rebase in-memory...
-        [1/2] Skipped commit (was already applied upstream): 62fc20d2 create test1.txt
-        [2/2] Committed as: fa466332 create test2.txt
-        branchless: processing 1 update: branch should-be-deleted
-        branchless: processing 2 rewritten commits
-        branchless: running command: <git-executable> checkout fa46633239bfa767036e41a77b67258286e4ddb9
-        In-memory rebase succeeded.
-        "###);
+            Attempting rebase in-memory...
+            [1/2] Skipped commit (was already applied upstream): 62fc20d2 create test1.txt
+            [2/2] Committed as: fa466332 create test2.txt
+            branchless: processing 1 update: branch should-be-deleted
+            branchless: processing 2 rewritten commits
+            branchless: running command: <git-executable> checkout fa46633239bfa767036e41a77b67258286e4ddb9
+            :
+            O 047b7ad7 (master) create test1.txt
+            |
+            @ fa466332 create test2.txt
+            In-memory rebase succeeded.
+            "###);
         }
 
         {
@@ -1185,7 +1234,6 @@ fn test_move_no_reapply_squashed_commits() -> eyre::Result<()> {
     }
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.detach_head()?;
     let test1_oid = git.commit_file("test1", 1)?;
@@ -1223,20 +1271,24 @@ fn test_move_no_reapply_squashed_commits() -> eyre::Result<()> {
                 "master",
             ])?;
             insta::assert_snapshot!(stderr, @r###"
-        Executing: git branchless hook-register-extra-post-rewrite-hook
-        branchless: processing 1 update: ref HEAD
-        branchless: processing 1 update: ref HEAD
-        branchless: processed commit: e7bcdd60 create test1.txt
-        Executing: git branchless hook-detect-empty-commit 62fc20d2a290daea0d52bdc2ed2ad4be6491010e
-        branchless: processing 1 update: ref HEAD
-        branchless: processed commit: 12d361aa create test2.txt
-        Executing: git branchless hook-detect-empty-commit 96d1c37a3d4363611c49f7e52186e189a04c531f
-        branchless: processing 4 rewritten commits
-        branchless: running command: <git-executable> checkout master
-        Switched to branch 'master'
-        branchless: processing checkout
-        Successfully rebased and updated master.
-        "###);
+            Executing: git branchless hook-register-extra-post-rewrite-hook
+            branchless: processing 1 update: ref HEAD
+            branchless: processing 1 update: ref HEAD
+            branchless: processed commit: e7bcdd60 create test1.txt
+            Executing: git branchless hook-detect-empty-commit 62fc20d2a290daea0d52bdc2ed2ad4be6491010e
+            branchless: processing 1 update: ref HEAD
+            branchless: processed commit: 12d361aa create test2.txt
+            Executing: git branchless hook-detect-empty-commit 96d1c37a3d4363611c49f7e52186e189a04c531f
+            branchless: processing 1 update: branch master
+            branchless: processing 4 rewritten commits
+            branchless: running command: <git-executable> checkout de4a1fe8f80b830d7d9a5b4adfd79fab3fcdc80c
+            branchless: processing 1 update: ref HEAD
+            HEAD is now at de4a1fe squashed test1 and test2
+            branchless: processing checkout
+            :
+            @ de4a1fe8 (master) squashed test1 and test2
+            Successfully rebased and updated refs/heads/master.
+            "###);
             insta::assert_snapshot!(stdout, @r###"
         branchless: running command: <git-executable> diff --quiet
         Calling Git for on-disk rebase...
@@ -1284,13 +1336,15 @@ fn test_move_no_reapply_squashed_commits() -> eyre::Result<()> {
         branchless: processing checkout
         "###);
             insta::assert_snapshot!(stdout, @r###"
-        Attempting rebase in-memory...
-        [1/2] Skipped now-empty commit: e7bcdd60 create test1.txt
-        [2/2] Skipped now-empty commit: 12d361aa create test2.txt
-        branchless: processing 2 rewritten commits
-        branchless: running command: <git-executable> checkout master
-        In-memory rebase succeeded.
-        "###);
+            Attempting rebase in-memory...
+            [1/2] Skipped now-empty commit: e7bcdd60 create test1.txt
+            [2/2] Skipped now-empty commit: 12d361aa create test2.txt
+            branchless: processing 2 rewritten commits
+            branchless: running command: <git-executable> checkout master
+            :
+            @ de4a1fe8 (master) squashed test1 and test2
+            In-memory rebase succeeded.
+            "###);
         }
 
         {
@@ -1313,7 +1367,6 @@ fn test_move_delete_checked_out_branch() -> eyre::Result<()> {
     }
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.run(&["checkout", "-b", "work"])?;
     let test1_oid = git.commit_file("test1", 1)?;
@@ -1350,22 +1403,26 @@ fn test_move_delete_checked_out_branch() -> eyre::Result<()> {
             git.run(&["checkout", "work"])?;
             let (stdout, stderr) = git.run(&["move", "--on-disk", "-b", "HEAD", "-d", "master"])?;
             insta::assert_snapshot!(stderr, @r###"
-        Executing: git branchless hook-register-extra-post-rewrite-hook
-        branchless: processing 1 update: ref HEAD
-        Executing: git branchless hook-skip-upstream-applied-commit 62fc20d2a290daea0d52bdc2ed2ad4be6491010e
-        Executing: git branchless hook-skip-upstream-applied-commit 96d1c37a3d4363611c49f7e52186e189a04c531f
-        branchless: processing 1 update: ref HEAD
-        branchless: processed commit: 012efd6e create test3.txt
-        Executing: git branchless hook-detect-empty-commit ffcba554683d83de283de084a7d3896e332bbcdb
-        branchless: processing 3 rewritten commits
-        branchless: processing 2 updates: branch more-work, branch work
-        branchless: running command: <git-executable> checkout 91c5ce63686889388daec1120bf57bea8a744bc2
-        Previous HEAD position was 012efd6 create test3.txt
-        branchless: processing 1 update: ref HEAD
-        HEAD is now at 91c5ce6 create test2.txt
-        branchless: processing checkout
-        Successfully rebased and updated work.
-        "###);
+            Executing: git branchless hook-register-extra-post-rewrite-hook
+            branchless: processing 1 update: ref HEAD
+            Executing: git branchless hook-skip-upstream-applied-commit 62fc20d2a290daea0d52bdc2ed2ad4be6491010e
+            Executing: git branchless hook-skip-upstream-applied-commit 96d1c37a3d4363611c49f7e52186e189a04c531f
+            branchless: processing 1 update: ref HEAD
+            branchless: processed commit: 012efd6e create test3.txt
+            Executing: git branchless hook-detect-empty-commit ffcba554683d83de283de084a7d3896e332bbcdb
+            branchless: processing 3 rewritten commits
+            branchless: processing 2 updates: branch more-work, branch work
+            branchless: running command: <git-executable> checkout 91c5ce63686889388daec1120bf57bea8a744bc2
+            Previous HEAD position was 012efd6 create test3.txt
+            branchless: processing 1 update: ref HEAD
+            HEAD is now at 91c5ce6 create test2.txt
+            branchless: processing checkout
+            :
+            @ 91c5ce63 (master) create test2.txt
+            |
+            o 012efd6e (more-work) create test3.txt
+            Successfully rebased and updated detached HEAD.
+            "###);
             insta::assert_snapshot!(stdout, @r###"
         branchless: running command: <git-executable> diff --quiet
         Calling Git for on-disk rebase...
@@ -1398,15 +1455,19 @@ fn test_move_delete_checked_out_branch() -> eyre::Result<()> {
         branchless: processing checkout
         "###);
             insta::assert_snapshot!(stdout, @r###"
-        Attempting rebase in-memory...
-        [1/3] Skipped commit (was already applied upstream): 62fc20d2 create test1.txt
-        [2/3] Skipped commit (was already applied upstream): 96d1c37a create test2.txt
-        [3/3] Committed as: 012efd6e create test3.txt
-        branchless: processing 2 updates: branch more-work, branch work
-        branchless: processing 3 rewritten commits
-        branchless: running command: <git-executable> checkout 91c5ce63686889388daec1120bf57bea8a744bc2
-        In-memory rebase succeeded.
-        "###);
+            Attempting rebase in-memory...
+            [1/3] Skipped commit (was already applied upstream): 62fc20d2 create test1.txt
+            [2/3] Skipped commit (was already applied upstream): 96d1c37a create test2.txt
+            [3/3] Committed as: 012efd6e create test3.txt
+            branchless: processing 2 updates: branch more-work, branch work
+            branchless: processing 3 rewritten commits
+            branchless: running command: <git-executable> checkout 91c5ce63686889388daec1120bf57bea8a744bc2
+            :
+            @ 91c5ce63 (master) create test2.txt
+            |
+            o 012efd6e (more-work) create test3.txt
+            In-memory rebase succeeded.
+            "###);
         }
 
         {
@@ -1464,7 +1525,6 @@ fn test_move_merge_commit() -> eyre::Result<()> {
         return Ok(());
     }
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.commit_file("test1", 1)?;
     git.run(&["checkout", "HEAD^"])?;
@@ -1564,21 +1624,32 @@ fn test_move_merge_commit() -> eyre::Result<()> {
         branchless: running command: <git-executable> rebase --continue
         "###);
             insta::assert_snapshot!(stderr, @r###"
-        Executing: git branchless hook-register-extra-post-rewrite-hook
-        branchless: processing 1 update: ref HEAD
-        branchless: processing 1 update: ref HEAD
-        branchless: processed commit: 96d1c37a create test2.txt
-        Executing: git branchless hook-detect-empty-commit fe65c1fe15584744e649b2c79d4cf9b0d878f92e
-        branchless: processing 1 update: ref refs/rewritten/merge-parent-4
-        branchless: processing 2 rewritten commits
-        branchless: running command: <git-executable> checkout 98b9119d16974f372e76cb64a3b77c528fc0b18b
-        Previous HEAD position was 96a2c4b Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
-        branchless: processing 1 update: ref HEAD
-        HEAD is now at 98b9119 create test3.txt
-        branchless: processing checkout
-        Successfully rebased and updated detached HEAD.
-        branchless: processing 1 update: ref refs/rewritten/merge-parent-4
-        "###);
+            Executing: git branchless hook-register-extra-post-rewrite-hook
+            branchless: processing 1 update: ref HEAD
+            branchless: processing 1 update: ref HEAD
+            branchless: processed commit: 96d1c37a create test2.txt
+            Executing: git branchless hook-detect-empty-commit fe65c1fe15584744e649b2c79d4cf9b0d878f92e
+            branchless: processing 1 update: ref refs/rewritten/merge-parent-4
+            branchless: processing 2 rewritten commits
+            branchless: running command: <git-executable> checkout 98b9119d16974f372e76cb64a3b77c528fc0b18b
+            Previous HEAD position was 96a2c4b Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
+            branchless: processing 1 update: ref HEAD
+            HEAD is now at 98b9119 create test3.txt
+            branchless: processing checkout
+            O f777ecc9 create initial.txt
+            |\
+            | @ 98b9119d create test3.txt
+            | |
+            | o 96a2c4be Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
+            |
+            O 62fc20d2 (master) create test1.txt
+            |
+            o 96d1c37a create test2.txt
+            |
+            o 96a2c4be Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
+            Successfully rebased and updated detached HEAD.
+            branchless: processing 1 update: ref refs/rewritten/merge-parent-4
+            "###);
         }
 
         {
@@ -1642,21 +1713,32 @@ fn test_move_merge_commit() -> eyre::Result<()> {
         branchless: running command: <git-executable> rebase --continue
         "###);
             insta::assert_snapshot!(stderr, @r###"
-        Executing: git branchless hook-register-extra-post-rewrite-hook
-        branchless: processing 1 update: ref HEAD
-        branchless: processing 1 update: ref HEAD
-        branchless: processed commit: 96d1c37a create test2.txt
-        Executing: git branchless hook-detect-empty-commit fe65c1fe15584744e649b2c79d4cf9b0d878f92e
-        branchless: processing 1 update: ref refs/rewritten/merge-parent-4
-        branchless: processing 2 rewritten commits
-        branchless: running command: <git-executable> checkout 98b9119d16974f372e76cb64a3b77c528fc0b18b
-        Previous HEAD position was 96a2c4b Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
-        branchless: processing 1 update: ref HEAD
-        HEAD is now at 98b9119 create test3.txt
-        branchless: processing checkout
-        Successfully rebased and updated detached HEAD.
-        branchless: processing 1 update: ref refs/rewritten/merge-parent-4
-        "###);
+            Executing: git branchless hook-register-extra-post-rewrite-hook
+            branchless: processing 1 update: ref HEAD
+            branchless: processing 1 update: ref HEAD
+            branchless: processed commit: 96d1c37a create test2.txt
+            Executing: git branchless hook-detect-empty-commit fe65c1fe15584744e649b2c79d4cf9b0d878f92e
+            branchless: processing 1 update: ref refs/rewritten/merge-parent-4
+            branchless: processing 2 rewritten commits
+            branchless: running command: <git-executable> checkout 98b9119d16974f372e76cb64a3b77c528fc0b18b
+            Previous HEAD position was 96a2c4b Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
+            branchless: processing 1 update: ref HEAD
+            HEAD is now at 98b9119 create test3.txt
+            branchless: processing checkout
+            O f777ecc9 create initial.txt
+            |\
+            | @ 98b9119d create test3.txt
+            | |
+            | o 96a2c4be Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
+            |
+            O 62fc20d2 (master) create test1.txt
+            |
+            o 96d1c37a create test2.txt
+            |
+            o 96a2c4be Merge commit 'fe65c1fe15584744e649b2c79d4cf9b0d878f92e' into HEAD
+            Successfully rebased and updated detached HEAD.
+            branchless: processing 1 update: ref refs/rewritten/merge-parent-4
+            "###);
         }
 
         {
@@ -1689,7 +1771,6 @@ fn test_move_orphaned_root() -> eyre::Result<()> {
     }
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.commit_file("test1", 1)?;
     git.commit_file("test2", 2)?;
@@ -1725,21 +1806,26 @@ fn test_move_orphaned_root() -> eyre::Result<()> {
         {
             let (stdout, stderr) = git.run(&["move", "--on-disk", "-d", "master"])?;
             insta::assert_snapshot!(stderr, @r###"
-        Executing: git branchless hook-register-extra-post-rewrite-hook
-        branchless: processing 1 update: ref HEAD
-        branchless: processing 1 update: ref HEAD
-        branchless: processed commit: 270b681e new root
-        Executing: git branchless hook-detect-empty-commit da90168b4835f97f1a10bcc12833140056df9157
-        branchless: processing 1 update: ref HEAD
-        branchless: processed commit: 70deb1e2 create test3.txt
-        Executing: git branchless hook-detect-empty-commit fc09f3d9f0b7370dc38e761e3730a856dc5025c2
-        branchless: processing 3 rewritten commits
-        branchless: processing 1 update: branch new-root
-        branchless: running command: <git-executable> checkout new-root
-        Switched to branch 'new-root'
-        branchless: processing checkout
-        Successfully rebased and updated new-root.
-        "###);
+            Executing: git branchless hook-register-extra-post-rewrite-hook
+            branchless: processing 1 update: ref HEAD
+            branchless: processing 1 update: ref HEAD
+            branchless: processed commit: 270b681e new root
+            Executing: git branchless hook-detect-empty-commit da90168b4835f97f1a10bcc12833140056df9157
+            branchless: processing 1 update: ref HEAD
+            branchless: processed commit: 70deb1e2 create test3.txt
+            Executing: git branchless hook-detect-empty-commit fc09f3d9f0b7370dc38e761e3730a856dc5025c2
+            branchless: processing 1 update: branch new-root
+            branchless: processing 3 rewritten commits
+            branchless: running command: <git-executable> checkout 70deb1e28791d8e7dd5a1f0c871a51b91282562f
+            branchless: processing 1 update: ref HEAD
+            HEAD is now at 70deb1e create test3.txt
+            branchless: processing checkout
+            :
+            O 96d1c37a (master) create test2.txt
+            |
+            @ 70deb1e2 (new-root) create test3.txt
+            Successfully rebased and updated refs/heads/new-root.
+            "###);
             insta::assert_snapshot!(stdout, @r###"
         branchless: running command: <git-executable> diff --quiet
         Calling Git for on-disk rebase...
@@ -1769,14 +1855,18 @@ fn test_move_orphaned_root() -> eyre::Result<()> {
         branchless: processing checkout
         "###);
             insta::assert_snapshot!(stdout, @r###"
-        Attempting rebase in-memory...
-        [1/2] Skipped now-empty commit: 270b681e new root
-        [2/2] Committed as: 70deb1e2 create test3.txt
-        branchless: processing 1 update: branch new-root
-        branchless: processing 2 rewritten commits
-        branchless: running command: <git-executable> checkout new-root
-        In-memory rebase succeeded.
-        "###);
+            Attempting rebase in-memory...
+            [1/2] Skipped now-empty commit: 270b681e new root
+            [2/2] Committed as: 70deb1e2 create test3.txt
+            branchless: processing 1 update: branch new-root
+            branchless: processing 2 rewritten commits
+            branchless: running command: <git-executable> checkout new-root
+            :
+            O 96d1c37a (master) create test2.txt
+            |
+            @ 70deb1e2 (new-root) create test3.txt
+            In-memory rebase succeeded.
+            "###);
         }
 
         {
@@ -1798,7 +1888,6 @@ fn test_move_no_extra_checkout() -> eyre::Result<()> {
     let git = make_git()?;
 
     git.init_repo()?;
-    git.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
     git.commit_file("test1", 1)?;
     git.detach_head()?;
@@ -1854,7 +1943,6 @@ fn test_move_dest_not_in_dag() -> eyre::Result<()> {
             ..Default::default()
         })?;
         cloned_repo.run(&["branchless", "init", "--main-branch", "other-branch"])?;
-        cloned_repo.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
 
         let (stdout, _stderr) = cloned_repo.run(&["move", "-d", "origin/master"])?;
         insta::assert_snapshot!(stdout, @r###"
@@ -1864,9 +1952,128 @@ fn test_move_dest_not_in_dag() -> eyre::Result<()> {
         branchless: processing 1 rewritten commit
         branchless: running command: <git-executable> checkout other-branch
         Your branch is up to date with 'origin/other-branch'.
+        :
+        @ 70deb1e2 (other-branch, remote origin/other-branch) create test3.txt
         In-memory rebase succeeded.
         "###);
     }
 
     Ok(())
 }
+
+#[test]
+fn test_move_abort_rebase_check_out_old_branch() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.commit_file("test1", 1)?;
+    git.detach_head()?;
+
+    git.run(&["checkout", "-b", "original"])?;
+    git.commit_file_with_contents("test2", 2, "test2 original contents")?;
+
+    git.run(&["checkout", "-b", "conflicting", "master"])?;
+    git.commit_file_with_contents("test2", 2, "test2 conflicting contents")?;
+
+    {
+        let (stdout, _stderr) = git.run_with_options(
+            &["move", "-d", "original", "--on-disk"],
+            &GitRunOptions {
+                expected_exit_code: 1,
+                ..Default::default()
+            },
+        )?;
+        let stdout = remove_rebase_lines(stdout);
+
+        insta::assert_snapshot!(stdout, @r###"
+        branchless: running command: <git-executable> diff --quiet
+        Calling Git for on-disk rebase...
+        branchless: running command: <git-executable> rebase --continue
+        CONFLICT (add/add): Merge conflict in test2.txt
+        "###);
+    }
+
+    git.run(&["rebase", "--abort"])?;
+
+    {
+        // Will output `HEAD` if `HEAD` is detached, which is not what we want.
+        let (stdout, _stderr) = git.run(&["rev-parse", "--abbrev-ref", "HEAD"])?;
+        insta::assert_snapshot!(stdout, @"conflicting");
+    }
+
+    Ok(())
+}
+
+/// For https://github.com/arxanas/git-branchless/issues/151
+#[test]
+fn test_move_orig_head_no_symbolic_reference() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo()?;
+    git.commit_file("test1", 1)?;
+    git.commit_file("test2", 2)?;
+
+    git.run(&["checkout", "-b", "foo"])?;
+    // Force `ORIG_HEAD` to be written to disk.
+    git.run(&["move", "-d", "HEAD^", "--on-disk"])?;
+    git.detach_head()?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        :
+        @ 96d1c37a (foo, master) create test2.txt
+        "###);
+    }
+
+    // Get `git reset` to write a new value to `ORIG_HEAD`. If `ORIG_HEAD` is a
+    // symbolic reference, it will write to the target reference, rather than
+    // overwriting `ORIG_HEAD`.
+    git.run(&["reset", "--hard", "HEAD^"])?;
+    git.run(&["reset", "--hard", "HEAD^"])?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        // `foo` should be unmoved here, rather than moved to
+        // `create test1.txt`.
+        insta::assert_snapshot!(stdout, @r###"
+        @ f777ecc9 create initial.txt
+        :
+        O 96d1c37a (foo, master) create test2.txt
+        "###);
+    }
+
+    Ok(())
+}
+
+#[test]
+fn test_move_standalone_no_create_gc_refs() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    git.init_repo_with_options(&GitInitOptions {
+        run_branchless_init: false,
+        ..Default::default()
+    })?;
+    let test1_oid = git.commit_file("test1", 1)?;
+    git.run(&["checkout", "HEAD^"])?;
+    git.commit_file("test2", 2)?;
+
+    let show_refs_output = {
+        let (stdout, _stderr) = git.run(&["show-ref"])?;
+        insta::assert_snapshot!(stdout, @"62fc20d2a290daea0d52bdc2ed2ad4be6491010e refs/heads/master
+");
+        stdout
+    };
+
+    git.run(&["branchless", "move", "-d", &test1_oid.to_string()])?;
+
+    {
+        let (stdout, _stderr) = git.run(&["show-ref"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        62fc20d2a290daea0d52bdc2ed2ad4be6491010e refs/heads/master
+        "###);
+        assert!(stdout == show_refs_output);
+    }
+
+    Ok(())
+}
