@@ -6,16 +6,16 @@ use std::sync::{Arc, Mutex};
 use crate::util::trim_lines;
 
 use branchless::commands::undo::testing::{select_past_event, undo_events};
+use branchless::core::dag::Dag;
+use branchless::core::effects::Effects;
 use branchless::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
 use branchless::core::formatting::Glyphs;
-use branchless::git::{Dag, GitRunInfo, Repo};
+use branchless::git::{GitRunInfo, Repo};
 use branchless::testing::{make_git, Git};
 use branchless::tui::testing::{screen_to_string, CursiveTestingBackend, CursiveTestingEvent};
-use branchless::tui::Effects;
 
 use cursive::event::Key;
 use cursive::CursiveRunnable;
-use os_str_bytes::OsStrBytes;
 
 fn run_select_past_event(
     repo: &Repo,
@@ -41,7 +41,7 @@ fn run_select_past_event(
     select_past_event(siv.into_runner(), &effects, repo, &dag, &mut event_replayer)
 }
 
-fn run_undo_events(git: &Git, event_cursor: EventCursor) -> eyre::Result<String> {
+fn run_undo_events(git: &Git, event_cursor: EventCursor) -> eyre::Result<(isize, String)> {
     let glyphs = Glyphs::text();
     let effects = Effects::new_suppress_for_test(glyphs.clone());
     let repo = git.get_repo()?;
@@ -50,48 +50,35 @@ fn run_undo_events(git: &Git, event_cursor: EventCursor) -> eyre::Result<String>
     let event_replayer = EventReplayer::from_event_log_db(&effects, &repo, &event_log_db)?;
     let input = "y";
     let mut in_ = input.as_bytes();
-    let out: Arc<Mutex<Vec<u8>>> = Default::default();
+    let stdout: Arc<Mutex<Vec<u8>>> = Default::default();
+    let stderr: Arc<Mutex<Vec<u8>>> = Default::default();
 
     let git_run_info = GitRunInfo {
         path_to_git: git.path_to_git.clone(),
-
-        // Ensure that nested calls to `git` are run under the correct environment.
-        // (Normally, the user will be running `git undo` from the correct directory
-        // already.)
         working_directory: repo.get_working_copy_path().unwrap().to_path_buf(),
-
-        // Normally, we want to inherit the user's environment when running external
-        // Git commands. However, for testing, we may have inherited variables which
-        // affect the execution of Git. In particular, `GIT_INDEX_FILE` is set to
-        // `.git/index` normally (which works for the test), but can be set to an
-        // absolute path when running `git commit -a`, and having these tests run as
-        // part of a commit hook.
-        env: std::env::vars_os()
-            .filter(|(k, _v)| !k.to_raw_bytes().starts_with(b"GIT_"))
-            .collect(),
+        env: git.get_base_env(0).into_iter().collect(),
     };
 
-    let result = undo_events(
+    let exit_code = undo_events(
         &mut in_,
-        &Effects::new_from_buffer_for_test(glyphs, &out),
+        &Effects::new_from_buffer_for_test(glyphs, &stdout, &stderr),
         &repo,
         &git_run_info,
         &mut event_log_db,
         &event_replayer,
         event_cursor,
     )?;
-    assert_eq!(result, 0);
 
-    let out = {
-        let mut buf = out.lock().unwrap();
+    let stdout = {
+        let mut buf = stdout.lock().unwrap();
         let mut result_buf = Vec::new();
         swap(&mut *buf, &mut result_buf);
         result_buf
     };
-    let out = String::from_utf8(out)?;
-    let out = git.preprocess_output(out)?;
-    let out = trim_lines(out);
-    Ok(out)
+    let stdout = String::from_utf8(stdout)?;
+    let stdout = git.preprocess_output(stdout)?;
+    let stdout = trim_lines(stdout);
+    Ok((exit_code, stdout))
 }
 
 #[test]
@@ -357,7 +344,7 @@ fn test_undo_hide() -> eyre::Result<()> {
     let event_cursor = event_cursor.unwrap();
 
     {
-        let stdout = run_undo_events(&git, event_cursor)?;
+        let (exit_code, stdout) = run_undo_events(&git, event_cursor)?;
         insta::assert_snapshot!(stdout, @r###"
             Will apply these actions:
             1. Create branch test1 at 62fc20d2 create test1.txt
@@ -366,6 +353,7 @@ fn test_undo_hide() -> eyre::Result<()> {
 
             Confirm? [yN] Applied 2 inverse events.
             "###);
+        assert_eq!(exit_code, 0);
     }
 
     {
@@ -413,7 +401,7 @@ fn test_undo_move_refs() -> eyre::Result<()> {
     let event_cursor = event_cursor.unwrap();
 
     {
-        let stdout = run_undo_events(&git, event_cursor)?;
+        let (exit_code, stdout) = run_undo_events(&git, event_cursor)?;
         insta::assert_snapshot!(stdout, @r###"
         Will apply these actions:
         1. Check out from 96d1c37a create test2.txt
@@ -422,9 +410,14 @@ fn test_undo_move_refs() -> eyre::Result<()> {
 
         3. Move branch master from 96d1c37a create test2.txt
                                 to 62fc20d2 create test1.txt
-        Confirm? [yN] branchless: running command: <git-executable> checkout --detach 62fc20d2a290daea0d52bdc2ed2ad4be6491010e
+        Confirm? [yN] branchless: running command: <git-executable> checkout 62fc20d2a290daea0d52bdc2ed2ad4be6491010e --detach
+        :
+        @ 62fc20d2 create test1.txt
+        |
+        O 96d1c37a (master) create test2.txt
         Applied 3 inverse events.
         "###);
+        assert_eq!(exit_code, 0);
     }
 
     {
@@ -633,7 +626,7 @@ fn test_undo_doesnt_make_working_dir_dirty() -> eyre::Result<()> {
         assert_eq!(stdout, "");
     }
     {
-        let stdout = run_undo_events(&git, event_cursor)?;
+        let (exit_code, stdout) = run_undo_events(&git, event_cursor)?;
         insta::assert_snapshot!(stdout, @r###"
         Will apply these actions:
         1. Check out from 62fc20d2 create test1.txt
@@ -646,9 +639,13 @@ fn test_undo_doesnt_make_working_dir_dirty() -> eyre::Result<()> {
                                 to f777ecc9 create initial.txt
         5. Delete branch foo at f777ecc9 create initial.txt
 
-        Confirm? [yN] branchless: running command: <git-executable> checkout --detach f777ecc9b0db5ed372b2615695191a8a17f79f24
+        Confirm? [yN] branchless: running command: <git-executable> checkout f777ecc9b0db5ed372b2615695191a8a17f79f24 --detach
+        @ f777ecc9 (foo) create initial.txt
+        |
+        O 62fc20d2 (bar, master) create test1.txt
         Applied 5 inverse events.
         "###);
+        assert_eq!(exit_code, 0);
     }
     {
         let (stdout, _stderr) = git.run(&["status", "--porcelain"])?;
@@ -713,3 +710,95 @@ fn test_git_bisect_produces_empty_event() -> eyre::Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_undo_garbage_collected_commit() -> eyre::Result<()> {
+    let git = make_git()?;
+
+    if !git.supports_reference_transactions()? {
+        return Ok(());
+    }
+    git.init_repo()?;
+
+    git.commit_file("test1", 1)?;
+    git.detach_head()?;
+    git.commit_file("test2", 2)?;
+    git.run(&["hide", "HEAD"])?;
+    git.run(&["checkout", "HEAD^"])?;
+
+    {
+        let (stdout, _stderr) = git.run(&["branchless", "gc"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        branchless: collecting garbage
+        branchless: 1 dangling reference deleted
+        "###);
+    }
+    git.run(&["gc", "--prune=now"])?;
+
+    {
+        let (stdout, _stderr) = git.run(&["smartlog"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        :
+        @ 62fc20d2 (master) create test1.txt
+        "###);
+    }
+
+    let screenshot1 = Default::default();
+    let event_cursor = run_select_past_event(
+        &git.get_repo()?,
+        vec![
+            CursiveTestingEvent::Event('p'.into()),
+            CursiveTestingEvent::Event('p'.into()),
+            CursiveTestingEvent::Event('p'.into()),
+            CursiveTestingEvent::Event('p'.into()),
+            CursiveTestingEvent::TakeScreenshot(Rc::clone(&screenshot1)),
+            CursiveTestingEvent::Event(Key::Enter.into()),
+        ],
+    )?;
+    insta::assert_snapshot!(screen_to_string(&screenshot1), @r###"
+    ┌───────────────────────────────────────────────────┤─Commit graph ├───────────────────────────────────────────────────┐
+    │:                                                                                                                     │
+    │O 62fc20d2 (master) create test1.txt                                                                                  │
+    │|                                                                                                                     │
+    │% 96d1c37a (manually hidden) <garbage collected>                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    │                                                                                                                      │
+    └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
+    ┌──────────────────────────────────────────────────────┤─Events ├──────────────────────────────────────────────────────┐
+    │Repo after transaction 7 (event 8). Press 'h' for help, 'q' to quit.                                                  │
+    │1. Hide commit <commit not available: 96d1c37a3d4363611c49f7e52186e189a04c531f>                                       │
+    │                                                                                                                      │
+    └──────────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
+    "###);
+
+    let event_cursor = event_cursor.unwrap();
+    {
+        let (exit_code, stdout) = run_undo_events(&git, event_cursor)?;
+        insta::assert_snapshot!(stdout, @r###"
+        Will apply these actions:
+        1. Check out from 62fc20d2 create test1.txt
+                       to <commit not available: 96d1c37a3d4363611c49f7e52186e189a04c531f>
+        2. Move branch master from 62fc20d2 create test1.txt
+                                to 62fc20d2 create test1.txt
+        3. Move branch master from 62fc20d2 create test1.txt
+                                to 62fc20d2 create test1.txt
+        Confirm? [yN] branchless: running command: <git-executable> checkout 96d1c37a3d4363611c49f7e52186e189a04c531f --detach
+        Failed to check out commit: 96d1c37a3d4363611c49f7e52186e189a04c531f
+        Applied 3 inverse events.
+        "###);
+        assert!(exit_code > 0);
+    }
+
+    Ok(())
+}
