@@ -1,3 +1,5 @@
+mod util;
+
 mod core {
     mod test_eventlog;
     mod test_gc;
@@ -5,6 +7,7 @@ mod core {
 }
 
 mod command {
+    mod test_amend;
     mod test_hide;
     mod test_init;
     mod test_move;
@@ -12,4 +15,5 @@ mod command {
     mod test_restack;
     mod test_smartlog;
     mod test_undo;
+    mod test_wrap;
 }
